
//  Linux Frame Buffer Device Configuration
//
//   Copyright 1995-1998 by Geert Uytterhoeven
//		       (Geert.Uytterhoeven@cs.kuleuven.ac.be)
//
//  --------------------------------------------------------------------------
//
//  This file is subject to the terms and conditions of the GNU General Public
//  License. See the file COPYING in the main directory of the Linux
//  distribution for more details.


%{

#define YYSTYPE		long

#include <stdio.h>
#include <string.h>

#include "database.h"


extern int y2lex(void);
extern void y2error(const char *fmt) __attribute__((noreturn));


static struct VideoMode {
    const char *name;
    // geometry
    __u32 xres;
    __u32 yres;
    __u32 vxres;
    __u32 vyres;
    __u32 depth;
    // acceleration
    __u32 accel_flags;
    // timings
    __u32 pixclock;
    __u32 left;
    __u32 right;
    __u32 upper;
    __u32 lower;
    __u32 hslen;
    __u32 vslen;
    // flags
    unsigned hsync : 1;
    unsigned vsync : 1;
    unsigned csync : 1;
    unsigned gsync : 1;
    unsigned extsync : 1;
    unsigned bcast : 1;
    unsigned laced : 1;
    unsigned dblscan : 1;
    // scanrates
    double drate;
    double hrate;
    double vrate;
} VideoMode;

static void ClearVideoMode(void)
{
    memset(&VideoMode, 0, sizeof(VideoMode));
    VideoMode.accel_flags = FB_ACCELF_TEXT;
}

static void AddVideoMode(const struct VideoMode *vmode)
{
    Geometry geometry;
    Format format;
    Virtual virtual_;

    geometry.xres = vmode->xres;
    geometry.yres = vmode->yres;
    geometry.pixclock = vmode->pixclock;
    geometry.left_margin = vmode->left;
    geometry.right_margin = vmode->right;
    geometry.upper_margin = vmode->upper;
    geometry.lower_margin = vmode->lower;
    geometry.hsync_len = vmode->hslen;
    geometry.vsync_len = vmode->vslen;
    if (vmode->hsync)
	geometry.sync |= FB_SYNC_HOR_HIGH_ACT;
    if (vmode->vsync)
	geometry.sync |= FB_SYNC_VERT_HIGH_ACT;
    if (vmode->csync)
	geometry.sync |= FB_SYNC_COMP_HIGH_ACT;
    if (vmode->gsync)
	geometry.sync |= FB_SYNC_ON_GREEN;
    if (vmode->extsync)
	geometry.sync |= FB_SYNC_EXT;
    if (vmode->bcast)
	geometry.sync |= FB_SYNC_BROADCAST;
    if (vmode->laced)
	geometry.vmode = FB_VMODE_INTERLACED;
    else if (vmode->dblscan)
	geometry.vmode = FB_VMODE_DOUBLE;
    else
	geometry.vmode = FB_VMODE_NONINTERLACED;

    format.bits_per_pixel = vmode->depth;
#warning    format.red = ...;
#warning    format.green = ...;
#warning    format.blue = ...;
#warning    format.transp = ...;
#warning    format.nonstd = ...;

    virtual_.xres_virtual = vmode->vxres;
    virtual_.yres_virtual = vmode->vyres;

    Videomode *node = new Videomode(vmode->name, geometry, format, &virtual_);
    node->accel_flags = vmode->accel_flags;
    Videomodes.Add(node);
}

%}

%start file

%token MODE GEOMETRY TIMINGS HSYNC VSYNC CSYNC GSYNC EXTSYNC BCAST LACED DOUBLE
       ENDMODE POLARITY BOOLEAN STRING NUMBER

%%

file	  : vmodes
	  ;


vmodes	  : // empty
	  | vmodes vmode
	  ;

vmode	  : MODE STRING geometry timings options ENDMODE
	    {
		VideoMode.name = (const char *)$2;
		AddVideoMode(&VideoMode);
		ClearVideoMode();
	    }
	  ;

geometry  : GEOMETRY NUMBER NUMBER NUMBER NUMBER NUMBER
	    {
		ClearVideoMode();
		VideoMode.xres = $2;
		VideoMode.yres = $3;
		VideoMode.vxres = $4;
		VideoMode.vyres = $5;
		VideoMode.depth = $6;
	    }
	  ;

timings	  : TIMINGS NUMBER NUMBER NUMBER NUMBER NUMBER NUMBER NUMBER
	    {
		VideoMode.pixclock = $2;
		VideoMode.left = $3;
		VideoMode.right = $4;
		VideoMode.upper = $5;
		VideoMode.lower = $6;
		VideoMode.hslen = $7;
		VideoMode.vslen = $8;
	    }
	  ;

options	  : // empty
	  | options hsync
	  | options vsync
	  | options csync
      | options gsync
	  | options extsync
	  | options bcast
	  | options laced
	  | options double
	  ;

hsync	  : HSYNC POLARITY
	    {
		VideoMode.hsync = $2;
	    }
	  ;

vsync	  : VSYNC POLARITY
	    {
		VideoMode.vsync = $2;
	    }
	  ;

csync	  : CSYNC POLARITY
	    {
		VideoMode.csync = $2;
	    }
	  ;

gsync	  : GSYNC POLARITY
	    {
		VideoMode.gsync = $2;
	    }
	  ;

extsync	  : EXTSYNC BOOLEAN
	    {
		VideoMode.extsync = $2;
	    }
	  ;

bcast	  : BCAST BOOLEAN
	    {
		VideoMode.bcast = $2;
	    }
	  ;

laced	  : LACED BOOLEAN
	    {
		VideoMode.laced = $2;
	    }
	  ;

double	  : DOUBLE BOOLEAN
	    {
		VideoMode.dblscan = $2;
	    }
	  ;

%%
