
//  Linux Frame Buffer Device Configuration
//
//   Copyright 1995-1999 by Geert Uytterhoeven <geert@linux-m68k.org>
//
//  --------------------------------------------------------------------------
//
//  This file is subject to the terms and conditions of the GNU General Public
//  License. See the file COPYING in the main directory of the Linux
//  distribution for more details.


%{

#define YYSTYPE		long

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "fb.h"
#include "database.h"

extern int yylex(void);
extern void yyerror(const char *fmt, ...)
    __attribute__((noreturn, format (printf, 1, 2)));


static GeometryNode *_GeometryNode = NULL;
static FormatNode *_FormatNode = NULL;

static Geometry _Geometry;
static Format _Format;
static Virtual _Virtual;

%}

%start file

%token OPENB CLOSEB GEOMETRY FORMAT MODE ALIAS
    HSYNC VSYNC CSYNC GSYNC EXTSYNC BCAST
    LACED DOUBLE GRAYSCALE NONSTD ACCEL VIRTUAL POLARITY BOOL STRING INT

%%

    //  Configuration file

file
    : sections
    ;

sections
    : // empty
    | sections section
    ;

section
    : geometry_section
    | format_section
    | mode_section
    | alias_section
    ;


    //  Screen Geometries

geometry_section
    : // empty
    | GEOMETRY named_geometry
    | GEOMETRY OPENB geometries CLOSEB
    ;

geometries
    : // empty
    | geometries named_geometry
    ;

named_geometry
    : STRING geometry
	{
	    const char *s = (const char *)$1;
	    GeometryNode *node = new GeometryNode(s, _Geometry);
	    Geometries.Add(node);
	    _Geometry.Clear();
	}
    ;

geometry
    : OPENB INT INT INT INT INT INT INT INT INT geometry_options CLOSEB
	{
	    _Geometry.xres = $2;
	    _Geometry.yres = $3;
	    _Geometry.pixclock = $4;
	    _Geometry.left_margin = $5;
	    _Geometry.right_margin = $6;
	    _Geometry.upper_margin = $7;
	    _Geometry.lower_margin = $8;
	    _Geometry.hsync_len = $9;
	    _Geometry.vsync_len = $10;
	}
    ;

geometry_options
    : // empty
    | geometry_options geometry_option
    ;

geometry_option
    : LACED
    | DOUBLE
    ;

geometry_ref
    : geometry_name
    | geometry

geometry_name
    : STRING
    | geometry
	{
	    const char *s = (const char *)$1;
	    _GeometryNode = Geometries.Find(s);
	    if (!_GeometryNode)
		yyerror("Unknown geometry `%s'\n", s);
	}
    ;


    //  Pixel Formats

format_section
    : // empty
    | FORMAT named_format
    | FORMAT OPENB formats CLOSEB
    ;

formats
    : // empty
    | formats named_format
    ;

named_format
    : STRING format
	{
	    const char *s = (const char *)$1;
	    FormatNode *node = new FormatNode(s, _Format);
	    Formats.Add(node);
	    _Format.Clear();
	}
    ;

format
    : OPENB INT format_options OPENB
	{
	    _Format.bits_per_pixel = $2;
	}
    ;

format_options
    : // empty
    | format_options format_option
    ;

format_option
    : GRAYSCALE
    | NONSTD INT
    ;

format_ref
    : format_name
    | format

format_name
    : STRING
    | format
	{
	    const char *s = (const char *)$1;
	    _FormatNode = Formats.Find(s);
	    if (!_FormatNode)
		yyerror("Unknown pixel format `%s'\n", s);
	}
    ;


    //  Virtual Screen Dimensions

virtual
    : VIRTUAL INT INT
	{
	    _Virtual.xres_virtual = $1;
	    _Virtual.yres_virtual = $2;
	}
    ;


    //  Video Modes

mode_section
    : // empty
    | MODE named_mode
    | MODE OPENB modes CLOSEB
    ;

modes
    : // empty
    | modes named_mode
    ;

named_mode
    : STRING OPENB geometry_ref format_ref mode_options CLOSEB
	{
	    const char *s = (const char *)$1;
	    Videomode *node = new Videomode(s,
				_GeometryNode ? *_GeometryNode : _Geometry,
				_FormatNode ? *_FormatNode : _Format);
	    Videomodes.Add(node);
	    _GeometryNode = NULL;
	    _Geometry.Clear();
	    _FormatNode = NULL;
	    _Format.Clear();
	}
    | STRING OPENB geometry_ref format_ref virtual mode_options CLOSEB
	{
	    const char *s = (const char *)$1;
	    Videomode *node = new Videomode(s,
				_GeometryNode ? *_GeometryNode : _Geometry,
				_FormatNode ? *_FormatNode : _Format,
				&_Virtual);
	    Videomodes.Add(node);
	    _GeometryNode = NULL;
	    _Geometry.Clear();
	    _FormatNode = NULL;
	    _Format.Clear();
	}
    ;

mode_options
    : // empty
    | mode_options mode_option
    ;

mode_option
    : ACCEL
    ;


    //  Aliases

alias_section
    : // empty
    | ALIAS named_alias
    | ALIAS OPENB aliases CLOSEB
    ;

aliases
    : // empty
    | aliases named_alias
    ;

named_alias
    : STRING STRING
	{
	    const char *s1 = (const char *)$1;
	    const char *s2 = (const char *)$2;
	    AliasNode *node = new AliasNode(s1, s2);
	    Aliases.Add(node);
	}
    ;

%%
